@@ -67,7 +67,7 @@ sub new {
 	$self->{callid} ||= md5_hex( time(), rand(2**32) );
 	$self->{cseq} ||= 0;
 	$self->{_transactions} = [];
-	$self->{_cseq_incoming} = 0;
+	$self->{_cseq_incoming} = undef;
 
 	# create tag on my side (to|from)
 	my $side = $self->{incoming} ? 'to':'from';
@@ -154,6 +154,8 @@ sub new_request {
 	my Net::SIP::Endpoint::Context $self = shift;
 	my ($method,$body,%args) = @_;
 
+	my $rsp40x = delete $args{resp40x};
+
 	my $request;
 	if ( ref($method)) {
 		# already a request object
@@ -200,6 +202,11 @@ sub new_request {
 	# overwrite any route header in request if we already learned a route
 	$request->set_header( route => $self->{route} ) if $self->{route};
 
+	if ( $rsp40x and $self->{auth} and $request->authorize( $rsp40x, $self->{auth} )) {
+		# update local cseq
+		($self->{cseq}) = $request->cseq =~m{(\d+)};
+	}
+
 	# create new transaction
 	my %trans = (
 		tid      => $request->tid,
@@ -334,6 +341,7 @@ sub handle_response {
 		my $r = $tr->{request};
 		my $auth = $self->{auth};
 		if ( $auth && $r->authorize( $response, $auth )) {
+			DEBUG(10,"retrying with authorization");
 			# found something to authorize
 			# redo request
 			# update local cseq from cseq in request
@@ -341,6 +349,7 @@ sub handle_response {
 			$endpoint->new_request( $r,$self );
 		} else {
 			# need user feedback
+			DEBUG(10,"no (usable) authorization data available");
 			invoke_callback($cb,@arg,EPERM,$code,$response,$leg,$from);
 		}
 		return;
@@ -460,8 +469,10 @@ sub handle_request {
 	my $cseq = $request->cseq;
 	my ($cseq_num) = $cseq=~m{^(\d+)};
 
-	DEBUG( 100,"method=%s cseq=%s/%s inc=%d", $request->method, $cseq_num,$cseq, $self->{_cseq_incoming} );
-	if ( $cseq_num < $self->{_cseq_incoming} ) {
+	DEBUG( 100,"method=%s cseq=%s/%s inc=%s", $request->method, $cseq_num,$cseq, 
+		defined($self->{_cseq_incoming}) ? $self->{_cseq_incoming} : '<undef>' );
+	if ( defined $self->{_cseq_incoming} 
+		and $cseq_num < $self->{_cseq_incoming} ) {
 		# must be an retransmit of an really old request, drop
 		DEBUG( 10,"retransmit of really old request? Dropping" );
 		return;
@@ -484,13 +495,15 @@ sub handle_request {
 
 	if ( $method eq 'ACK' || $method eq 'CANCEL' ) {
 		# must be have same cseq_num as last request, otherwise drop
-		if ( $cseq_num != $self->{_cseq_incoming} ) {
+		if ( defined $self->{_cseq_incoming} 
+			and $cseq_num != $self->{_cseq_incoming} ) {
 			DEBUG( 10,"received $method for unreceived INVITE: $cseq_num|$self->{_cseq_incoming}" );
 			return;
 		}
 	} else {
 		# cannot have the same cseq_num as last request
-		if ( $cseq_num == $self->{_cseq_incoming} ) {
+		if ( defined $self->{_cseq_incoming}
+			and $cseq_num == $self->{_cseq_incoming} ) {
 			DEBUG( 10,"reused cseq for $method. DROP" );
 			return;
 		}
