@@ -125,6 +125,11 @@ sub create_ack {
 	# ACK uses cseq from request
 	$self->cseq =~m{(\d+)};
 	my $cseq = "$1 ACK";
+	my %auth;
+	for (qw(authorization proxy-authorization)) {
+		my $v = scalar($self->get_header($_)) or next;
+		$auth{$_} = $v;
+	}
 	my $header = {
 		'call-id' => scalar($self->get_header('call-id')),
 		from      => scalar($self->get_header('from')),
@@ -133,6 +138,7 @@ sub create_ack {
 		via       => [ ($self->get_header( 'via' ))[0] ],
 		route     => [ $self->get_header( 'route' ) ],
 		cseq      => $cseq,
+		%auth,
 	};
 	return Net::SIP::Request->new( 'ACK',$self->uri,$header );
 }
@@ -148,6 +154,11 @@ sub create_cancel {
 	# CANCEL uses cseq from request
 	$self->cseq =~m{(\d+)};
 	my $cseq = "$1 CANCEL";
+	my %auth;
+	for (qw(authorization proxy-authorization)) {
+		my $v = scalar($self->get_header($_)) or next;
+		$auth{$_} = $v;
+	}
 	my $header = {
 		'call-id' => scalar($self->get_header('call-id')),
 		from      => scalar($self->get_header('from')),
@@ -156,6 +167,7 @@ sub create_cancel {
 		via       => [ ($self->get_header( 'via' ))[0] ],
 		route     => [ $self->get_header( 'route' ) ],
 		cseq      => $cseq,
+		%auth
 	};
 	return Net::SIP::Request->new( 'CANCEL',$self->uri,$header );
 }
@@ -276,6 +288,7 @@ sub authorize {
 
 				# 3.2.2.1
 				if ( $h->{qop} ) {
+					$h->{qop} = 'auth'; # in case it was 'auth,auth-int'
 					my $nc = $digest{nc} = '00000001';
 					my $cnonce = $digest{cnonce} = sprintf("%08x",rand(2**32));
 					$digest{qop} = $h->{qop};
@@ -312,9 +325,12 @@ sub authorize {
 
 	return if !$auth; # no usable authenticate headers found
 
-	# increase cseq, because this will be a new request, not a retransmit
+	my ($rseq) = $response->cseq =~m{^(\d+)};
 	$self->cseq =~m{^(\d+)(.*)};
-	$self->set_header( cseq => ($1+1).$2 );
+	if ( defined $1 and $1 <= $rseq ) {
+		# increase cseq, because this will be a new request, not a retransmit
+		$self->set_header( cseq => ($rseq+1).$2 );
+	}
 
 	return 1;
 }
